Type Enemy
	Field spr:Sprite
	Field speed:Float
	Field dirx:Float
	Field diry:Float
End Type

' make a module type that acts as a game
Type game Extends AppModule
	
	Const WAIT_TO_START:Int = 0
	Const IN_GAME:Int = 1
	Const AFTER_GAME:Int = 2
	
	Field currentLevel:Int = 1
	Field totalFailTime:Int		' time in milliseconds
	Field measuredLevelTime:Int	' time until player hit mouse button
	
	Field enemies:TList
	Field enemyMasterSprite:MasterSprite
	
	Field playerSprite:MasterSprite
	
	Field background:MasterSprite
	
	Field playerCollides:Int = False
	
	Field playerScore:Int
	Field timeBonus:Int
	Field collisionBonus:Int
	Field nrCollisions:Int
	
	Field internalState:Int = WAIT_TO_START
	
	Field messageShowing:Int = False

	
	' simple create function
	Function Create ( resourceFile:String )
		Local g:game = New game
		g.Initialise( resourceFile, APP_GAME )
	End Function 
	
	' a new method, called once when the module starts	
	Method Start()
		Super.Start()
		
		PlayGlobalSong("drifted")
		background = GetSprite("background")
		playerSprite = GetSprite("player")
		enemyMasterSprite = GetSprite("enemy")
		enemies = New TList
		totalFailTime = 0
		measuredLevelTime = 0
		currentLevel = 0
		playerCollides = False
		messageShowing = False
		nrCollisions = 0
		playerScore = 0
		FlushMouse()
		
	End Method
	
	Method FadeOutComplete()
		AddHighScore( 0, playerScore ) 
		EndModule()
	End Method 
	
	' an update method that's called every 'tick'
	Method Update( timeMS:Int )
		Super.Update( timeMS )
		
		If ( KeyHit( KEY_ESCAPE ) )
			StartFadeOut()
		End If
		
		If internalState = WAIT_TO_START
			If Not messageShowing
				ShowStartLevelMessage()
			End If
			If MouseHit(1)
				nextLevel()
			End If
		End If
		
		' position our player
		Local px:Int = MouseX()
		Local py:Int = MouseY()
		If px - (playerSprite.Image().width / 2) < 0
			px = playerSprite.Image().width / 2
		End If
		If px + (playerSprite.Image().width / 2) > SCREEN_WIDTH
			px = SCREEN_WIDTH - playerSprite.Image().width / 2
		End If
		If py - (playerSprite.Image().Height / 2) < 0
			py = playerSprite.Image().Height / 2
		End If
		If py + (playerSprite.Image().Height / 2) > SCREEN_HEIGHT
			py = SCREEN_HEIGHT - playerSprite.Image().Height / 2
		End If
		playerSprite.SetPosition(px, py)
		
		' move all our enemies around the screen
		For Local e:Enemy = EachIn enemies
			Local newx:Float = e.spr.GetX() + e.dirx * e.speed * timeMS
			Local newy:Float = e.spr.GetY() + e.diry * e.speed * timeMS
			If newx < 32 Or newx > 768
				e.dirx = -e.dirx
				newx = e.spr.GetX()
			End If
			If newy < 32 Or newy > 568
				e.diry = -e.diry
				newy = e.spr.GetY()
			End If
			e.spr.SetPosition(newx, newy)
			e.spr.Update(timeMS)
		Next 
		
		' check for collisions and modify fail time only when state is IN_GAME
		If internalState = IN_GAME
			measuredLevelTime:+timeMS
			Local collision:Int = False
			px = playerSprite.GetX()
			py = playerSprite.GetY()
			Local pr:Float = playerSprite.GetRotation()
			For Local e:Enemy = EachIn enemies
				Local ex:Int = e.spr.GetX()
				Local ey:Int = e.spr.GetY()
				Local er:Float = e.spr.GetRotation()
				If ImagesCollide2(e.spr.Image(), ex, ey, 0, er, 1, 1, playerSprite.Image(), px, py, 0, pr, 1, 1)
					collision = True
					PlaySoundByName("collide")
				EndIf
			Next
			If collision = True
				playerCollides = True
			Else
				' increase nr of collisions if we had a collisions but now don't have one anymore
				If playerCollides = True
					nrCollisions:+1
					totalFailTime:+500 ' 2 collisions = 1 second
				End If
				playerCollides = False
			End If
			
			' check for game over if totalFailTime > 10 seconds
			If totalFailTime > 10000 Or MouseHit(1)
				' level done, baby
				internalState = AFTER_GAME
				' do the score calculation stuff
				calculateScore()
			End If
		End If
		
		If internalState = AFTER_GAME
			If Not messageShowing
				ShowEndLevelMessage()
			End If
			If MouseHit(1)
				If totalFailTime < 10000
					nextLevel()
				Else
					StartFadeOut()
				End If
			End If
		End If
		
	End Method

	
	Method nextLevel()
		internalState = IN_GAME
		measuredLevelTime = 0
		nrCollisions = 0
		currentLevel:+1
		While enemies.Count() < currentLevel
			Local en:Enemy = New Enemy
			en.spr = enemyMasterSprite.CreateInstance()
			en.spr.SetPosition(Rand(32, 800 - 32), Rand(32, 600 - 32))
			en.speed = Rnd(0.5, 0.8)
			en.dirx = Rnd(-1, 1)
			en.diry = Rnd(-1, 1)
			enemies.AddLast(en)
		End While
		ClearMessage()
	End Method
	
	
	Method calculateScore()
		Local diff:Int
		timeBonus = 0
		collisionBonus = 0
		diff = Abs(10000 - measuredLevelTime)
		If diff < 2000
			timeBonus = 1000
		End If
		If diff < 1000
			timeBonus = 2000
		End If
		If diff < 500
			timeBonus = 4000
		End If
		If diff < 100
			timeBonus = 8000
		End If
		
		If nrCollisions < 10
			collisionBonus = 1000
		End If
		If nrCollisions < 5
			collisionBonus = 2000
		End If
		If nrCollisions <= 3
			collisionBonus = 4000
		End If
		If nrCollisions = 1
			collisionBonus = 6000
		End If
		If nrCollisions = 0
			collisionBonus = 8000
		End If
		
		playerScore:+(timeBonus + collisionBonus)
		totalFailTime:+diff
	End Method
	
	
	' a draw method that's called every tick
	Method Draw()
		background.Draw()

		' draw the player
		playerSprite.Draw()

		' draw the enemies
		For Local e:Enemy = EachIn enemies
			e.spr.Draw()
		Next 

		
		DrawHUD()
		
		' draw the message window if required
		If messageShowing
			DrawGUI()		
		End If
		
		Super.Draw()
	End Method
	
	Method DrawHUD()
		SetScale(1.0, 1.0)
		SetRotation(0)
		GuiDrawText("Fail time: " + Decimals((Float(totalFailTime) / 1000.0), 3), 10, 10)
		GuiDrawText("Collisions: " + nrCollisions, 350, 10)
		GuiDrawText("Score: " + playerScore, 600, 10)
	End Method

	
	Method ShowStartLevelMessage()
		Local title:String
		Local lines:String[]
		
		title = "Get ready!"
		lines = New String[8]
		lines[0] = "Left click to start level " + (currentLevel + 1) + "!"
		lines[1] = "Left click again when you think"
		lines[2] = "that 10 seconds are over!"
		lines[3] = ""
		lines[4] = "Avoid everything to keep your fail time low!"
		lines[5] = ""
		lines[6] = ""
		lines[7] = ""
		ShowMessage(title, lines)
	End Method
	
	Method ShowEndLevelMessage()
		Local title:String
		Local lines:String[]
		
		title = "Level finished!"
		lines = New String[8]
		If totalFailTime > 10000
			lines[0] = "Your total fail time is up!"
			lines[1] = ""
			lines[2] = ""
			lines[3] = "Your final score: " + playerScore
			lines[4] = ""
			lines[5] = ""
			lines[6] = ""
			lines[7] = "Click mouse to return to menu"
		Else
			lines[0] = "Level time to guess: 10 seconds"
			lines[1] = "You guessed: " + Decimals((Float(measuredLevelTime) / 1000.0),3) + " seconds"
			lines[2] = "Resulting time bonus: " + timeBonus
			lines[3] = "Nr of collisions: " + nrCollisions
			lines[4] = "Resulting collision bonus: " + collisionBonus
			lines[5] = "Your new score: " + playerScore
			lines[6] = ""
			lines[7] = "Click mouse to start next level"
		End If
		ShowMessage(title, lines)
	End Method

	Method ShowMessage(strTitle:String, lines:String[])
		GuiShowWindow("messages")
		GuiSetControlCaptionText("messages", "title", strTitle)
		For Local x:Int = 0 To 7
			Local id:String = "line" + x
			GuiSetControlCaptionText("messages", id, lines[x])
		Next
		messageShowing = True
	End Method
	
	Method ClearMessage()
		GuiHideWindow("messages")
		messageShowing = False
	End Method
	
	Method Decimals:String(A:Double, n:Int)
		If n < 0 n = 0
		Local s:String = String(A)
		Local P:Int = s.Find(".")
		Local u:String = Mid(s, P + 2, n + 1)
		Local c:Int = Mid(u, n + 1, 1) >= "5"
		Local B:Int = Int(Mid(u, 1, n)) + c
		If n > 0
			Return Mid(s, 1, P) + "." + B
		Else
			Return Int(Mid(s, 1, P)) + c
		EndIf
	EndMethod
	
End Type
